@@ -700,7 +700,7 @@ int LsapiConn::processRespHeader(char *pEnd, int &status)
                 swapIntEndian(&m_respInfo.m_cntHeaders);
                 swapIntEndian(&m_respInfo.m_status);
             }
-            if (m_respInfo.m_status && m_respInfo.m_status != 200)
+            if (m_respInfo.m_status)
             {
                 int code;
                 code = HttpStatusCode::getInstance().codeToIndex(m_respInfo.m_status);
@@ -748,7 +748,7 @@ int LsapiConn::processRespHeader(char *pEnd, int &status)
                         *pHeaderEnd = 0;
                         if (HttpCgiTool::processHeaderLine(
                                 getConnector(),
-                                m_pRespHeaderProcess, pHeaderEnd, status) == -1)
+                                m_pRespHeaderProcess, pHeaderEnd) == -1)
                             return LS_FAIL;
                         m_pRespHeaderProcess += len;
                     }
