@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -79,8 +79,13 @@ ClientInfo::ClientInfo()
 #ifdef USE_IP2LOCATION
     , m_pLocInfo(NULL)
 #endif
+    , m_tmOverLimit(0)
+    , m_sslNewConn(0)
     , m_iCaptchaTries( 0 )
     , m_iAllowedBotHits( 0 )
+    , m_iHits(0)
+    , m_lastConnect(0)
+    , m_iAccess(0)
 {
 #if 0
     m_pShmClient = NULL;
