@@ -28,6 +28,7 @@ enum
 
 
 #include <http/httpheader.h>
+#include <http/httpmethod.h>
 #include <http/httpstatuscode.h>
 
 #include "httpvhost.h"
@@ -55,7 +56,7 @@ enum
 #define REWRITE_PERDIR          (1<<8)
 #define REWRITE_QSD             (1<<9)
 #define REWRITE_CACHE_CONF      (1<<10)
-#define CACHE_DECOMPRESS        (1<<11)
+#define REBUILD_COOKIE_UPKHDR   (1<<11)
 #define NO_RESP_BODY            (1<<12)
 #define IS_ERROR_PAGE           (1<<13)
 #define LSCACHE_FRONTEND        (1<<14)
@@ -93,7 +94,6 @@ enum
     URL_GENERIC,
     URL_FAVICON,
     URL_ROBOTS_TXT,
-    URL_WELL_KNOWN,
     URL_ACME_CHALLENGE,
     URL_CAPTCHA,
 };
@@ -184,7 +184,7 @@ public:
     void reset()
     {   clear(); m_iSessIdx = 0;    }
 
-    void cookieClassify(cookieval_t *pCookieEntry,
+    int cookieClassify(cookieval_t *pCookieEntry,
                         const char *pCookies, int nameLen,
                         const char *pVal, int valLen);
 
@@ -226,7 +226,7 @@ private:
     KVPairArray         m_unknHeaders;
 
     //Comment:The order of the below 3 varibles should NOT be changed!!!
-    uint16_t            m_commonHeaderLen[HttpHeader::H_TE];
+    short               m_commonHeaderLen[HttpHeader::H_TE];
     int                 m_commonHeaderOffset[HttpHeader::H_TE];
     int                 m_headerIdxOff;
     int                 m_reqLineOff;
@@ -251,7 +251,7 @@ private:
     char                m_method;
     char                m_iUserAgentType;
     short               m_iUrlType;
-    short               m_iCfRealIpHeader;
+    short               m_iCfIpHeader;
     int                 m_iEnvCount;
 
     unsigned short      m_ver;
@@ -272,6 +272,7 @@ private:
     //const HttpContext * m_pHTAContext;
     const HttpContext  *m_pFMContext;
     VMemBuf            *m_pReqBodyBuf;
+    UnpackedHeaders    *m_pUpkdHeaders;
     HttpRange          *m_pRange;
     HttpVHost          *m_pVHost;
     const char         *m_pForcedType;
@@ -298,7 +299,6 @@ private:
     AutoStr2            m_lastStatPath;
     struct stat         m_lastStat;
     int                 m_lastStatRes;
-    AutoStr2            m_headerLeftOver;
 
     static_file_data_t *m_pUrlStaticFileData;
 
@@ -341,7 +341,7 @@ private:
         m_commonHeaderLen[index] = len;
     }
     key_value_pair *getCurHeaderIdx()
-    {   return m_unknHeaders.getObj(m_unknHeaders.getSize() - 1);   }
+    {   return m_unknHeaders.getObj(m_unknHeaders.size() - 1);   }
     key_value_pair *newUnknownHeader();
 
     key_value_pair *newKeyValueBuf();
@@ -685,12 +685,12 @@ public:
 
     int  getUnknownHeaderCount() const
     {
-        return m_unknHeaders.getSize();
+        return m_unknHeaders.size();
     }
     const char *getUnknownHeaderByIndex(int idx, int &keyLen,
                                         const char *&pValue, int &valLen) const;
-    char isCfRealIpSet() const          {   return m_iCfRealIpHeader;   }
-    const char *getCfRealIpHeader(char *name, int &len);
+    char isCfIpSet() const                      {   return m_iCfIpHeader;   }
+    const char *getCfIpHeader(int &len);
 
     void setCrypto(HioCrypto *p)        {    m_pCrypto = p;         }
     HioCrypto *getCrypto() const        {   return m_pCrypto;       }
@@ -886,6 +886,8 @@ public:
     cookieval_t *getCookie(const char *pName, int nameLen);
     cookieval_t *insertCookieIndex(const char *pName, int nameLen);
     int parseCookies();
+    int parseCookies(const char *cookies, const char *end);
+    int parseOneCookie(const char *pCookies, const char *pValEnd);
     int copyCookieHeaderToBufEnd(int oldOff, const char *pCookie,
                                  int cookieLen);
     CookieList  &getCookieList() { return   m_cookies; }
@@ -904,9 +906,6 @@ public:
     void appendReqHeader( const char *pName, int iNameLen,
                           const char *pValue, int iValLen);
 
-    void saveHeaderLeftOver();
-    void restoreHeaderLeftOver();
-
     void classifyUrl();
 
     const Recaptcha *getRecaptcha() const;
