@@ -133,13 +133,15 @@ enum HSPState
 #define HSF_HANDLER_WRITE_SUSPENDED (1<<3)
 #define HSF_RESP_FLUSHED            (1<<4)
 #define HSF_REQ_BODY_DONE           (1<<5)
+
 #define HSF_AIO_READING             (1<<6)
 #define HSF_ACCESS_LOG_OFF          (1<<7)
 #define HSF_NO_ERROR_PAGE           (1<<8)
-
-#define HSF_HOOK_SESSION_STARTED    (1<<10)
-#define HSF_NO_CONN_TIMEOUT         (1<<11)
+#define HSF_EXEC_EXT_CMD            (1<<9)
+#define HSF_EXEC_POPEN              (1<<10)
+#define HSF_HOOK_SESSION_STARTED    (1<<11)
 #define HSF_NO_ABORT                (1<<12)
+
 #define HSF_RECV_RESP_BUFFERED      (1<<13)
 #define HSF_SEND_RESP_BUFFERED      (1<<14)
 
@@ -168,8 +170,6 @@ enum HSPState
 
 //Start flag2
 #define HSF2_IS_HTTP2               (1<<0)
-#define HSF2_EXEC_EXT_CMD           (1<<9)
-#define HSF2_EXEC_POPEN             (1<<10)
 
 
 typedef int (*SubSessionCb)(HttpSession *pSubSession, void *param,
@@ -184,7 +184,7 @@ class HttpSession
     , public InputStream
     , public HioHandler
     , public ls_lfnodei_t
-    , public LogSession
+    , virtual public LogSession
 {
     HttpReq               m_request;
     HttpResp              m_response;
@@ -520,8 +520,8 @@ public:
     void wantWrite(int want)    {   getStream()->wantWrite(want);     }
     void switchWriteToRead()    {    getStream()->switchWriteToRead();  };
 
-    void suspendEventNotify()   {    getStream()->suspendEventNotify(); };
-    void resumeEventNotify()    {    getStream()->resumeEventNotify();  };
+    void suspendEventNotify()   {    getStream()->suspendNotify(); };
+    void resumeEventNotify()    {    getStream()->resumeNotify();  };
 
     off_t getBytesRecv() const  {   return getStream()->getBytesRecv();    }
     off_t getBytesSent() const  {   return getStream()->getBytesSent();    }
@@ -565,7 +565,6 @@ public:
     long getReqTime() const {   return m_lReqTime;  }
     int32_t getReqTimeUs() const    {   return m_iReqTimeUs;    }
 
-    void bytesSent(int bytes);
     int writeRespBodyDirect(const char *pBuf, int size);
     int writeRespBody(const char *pBuf, int len);
 
@@ -665,7 +664,7 @@ public:
     GzipBuf *getGzipBuf() const     {   return getResp()->getGzipBuf();     }
     void setGzipBuf(GzipBuf *pGzip) {   getResp()->setGzipBuf(pGzip);       }
 
-    int execExtCmd(const char *pCmd, int len, int mode = 0);
+    int execExtCmd(const char *pCmd, int len, int mode = HSF_EXEC_EXT_CMD);
 
     int handlerProcess(const HttpHandler *pHandler);
     int getParsedScript(SsiScript *&pScript);
@@ -729,7 +728,6 @@ public:
     void setSendFileBeginEnd(off_t start, off_t end);
     void prepareHeaders();
     int sendRespHeaders();
-    int addExpiresHeader();
     void addLocationHeader();
 
     void setAccessLogOff()      {   setFlag(HSF_ACCESS_LOG_OFF);    }
