@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -105,8 +105,9 @@ public:
         if ((int)(len + m_lineBuf.size()) > size)
             return LS_FAIL;
         m_lineBuf += m_data[0].substr(0, len);
+        bool wasData = p != NULL;
         m_data[0].erase(0, len);
-        if (p)
+        if (wasData)
         {
             len = m_lineBuf.size();
             memmove(pBuf, m_lineBuf.c_str(), len);
